#include <os.h>
#include "utils.h"
#include "output.h"

inline void insert(char* buffer, int position, char ins){
    memmove(buffer + position + 1, buffer + position, strlen(buffer) - position + 1);
    *(buffer + position) = ins;
}

inline void insertstr(char* buffer, int position, char* ins){
    memmove(buffer + position + strlen(ins), buffer + position, strlen(buffer) - position + 1);//move everything including null char!
    memcpy(buffer + position, ins, strlen(ins));
}

inline char delchar(char* buffer, int position){
    char deletedchar = buffer[position-1];
    if(position != strlen(buffer))
        memmove(buffer + position - 1, buffer + position, strlen(buffer) - position + 1);
    else
        *(buffer + position - 1) = '\0';
    return deletedchar;
}

inline void delselection(char* buffer, int startoffset, int endoffset){
    if(buffer[endoffset] != '\0')
        memmove(buffer+startoffset, buffer+endoffset, strlen(buffer) - endoffset + 1);//move everything after selection (including null character)
    else
        buffer[startoffset] = '\0';
}


inline int strprevnewl(char* textbuffer, int pos){//returns position_of_last_newline_character_before_pos + 1 or zero if not found
    pos--;
    while(textbuffer[pos] != '\n' && pos >= 0)
        pos--;
    return ++pos;
}

int getw(char* textbuffer, int pos){
    int tmppos = strprevnewl(textbuffer, pos);
    int w = 0;
    while(tmppos <= pos){
        for(w = 0; textbuffer[tmppos] != '\n' && w < SCREEN_WIDTH/CHAR_WIDTH; tmppos++){
            if(tmppos >= pos)//how many characters into the line is pos?
                break;
            if(textbuffer[tmppos] == '\t')
                w += TAB_WIDTH - (w % TAB_WIDTH);
            else
                w++;
        }
        if(textbuffer[tmppos] == '\n'){
            if(tmppos >= pos)
                break;
            tmppos++;
        }
        if(tmppos >= pos)
            break;
    }
    if(w >= SCREEN_WIDTH/CHAR_WIDTH)
        return 0;
    return w;
}

int getnum(char* textbuffer, int pos){
    int tmppos = strprevnewl(textbuffer, pos);
    int w = 0;
    int num = 0;
    while(tmppos <= pos){
        num = 0;
        for(w = 0; textbuffer[tmppos] != '\n' && w < SCREEN_WIDTH/CHAR_WIDTH; tmppos++){
            if(tmppos >= pos)//how many characters into the line is pos?
                break;
            if(textbuffer[tmppos] == '\t')
                w += TAB_WIDTH - (w % TAB_WIDTH);
            else
                w++;
            num++;
        }
        if(textbuffer[tmppos] == '\n'){
            if(tmppos >= pos)
                break;
            tmppos++;
        }
        if(tmppos >= pos)
            break;
    }
    if(w >= SCREEN_WIDTH/CHAR_WIDTH)
        return 0;
    return num;
}

inline int currentline(char* textbuffer, int pos){
    return pos - getnum(textbuffer, pos);
}

int prevline(char* textbuffer, int pos){
    int tmppos;
    int w;
    int linestart = 0;
    pos = currentline(textbuffer, pos);
    if(pos == 0)
        return 0;
    pos--;
    tmppos = strprevnewl(textbuffer, pos);
    while(tmppos <= pos){
        linestart = tmppos;//save start of line
        for(w = 0; textbuffer[tmppos] != '\n' && w < SCREEN_WIDTH/CHAR_WIDTH && textbuffer[tmppos] != '\0'; tmppos++){
            if(textbuffer[tmppos] == '\t')
                w += TAB_WIDTH - (w % TAB_WIDTH);
            else
                w++;
        }
        if((textbuffer[tmppos] == '\n' || textbuffer[tmppos] == '\0') && w < SCREEN_WIDTH/CHAR_WIDTH)
            tmppos++;
    }
    return linestart;
}

int nextline(char* textbuffer, int pos){
    int tmppos = strprevnewl(textbuffer, pos);//start at first character after newline
    int w;
    while(tmppos <= pos){
        for(w = 0; textbuffer[tmppos] != '\n' && w < SCREEN_WIDTH/CHAR_WIDTH && textbuffer[tmppos] != '\0'; tmppos++){
            if(textbuffer[tmppos] == '\t')
                w += TAB_WIDTH - (w % TAB_WIDTH);
            else
                w++;
        }
        if(textbuffer[tmppos] == '\0')//return 0 if in last line
            return 0;
        if(textbuffer[tmppos] == '\n' && w < SCREEN_WIDTH/CHAR_WIDTH)
            tmppos++;
    }
    return tmppos;
}

char* getline(char* textbuffer, int pos, int cursorscreenrow){//returns pointer to first character of screen if position in row=cursorscreenrow on screen
    while(cursorscreenrow--)
        pos = prevline(textbuffer, pos);
    pos = currentline(textbuffer, pos);
    return textbuffer + pos;
}
